/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/acceptable_keyset.hh"

#include <algorithm>
#include <type_traits>

using namespace Fred::Akm;

const DomainUuid& Fred::Akm::domain_uuid(const AcceptableKeyset& src)
{
    static_assert(std::is_same<decltype(src.first)&, decltype(domain_uuid(src))>::value, "incompatible types");
    return src.first;
}

void Fred::Akm::append(AcceptableKeysets& collection, const AcceptableKeyset& item)
{
    const auto iter = collection.find(item.first);
    if (iter == cend(collection))
    {
        collection.insert(item);
        return;
    }
    if (iter->second.domain_fqdn != item.second.domain_fqdn)
    {
        throw std::runtime_error{"one uuid two different fqdns"};
    }
    iter->second.keyset.insert(cbegin(item.second.keyset), cend(item.second.keyset));
}

void Fred::Akm::merge(AcceptableKeysets& dst, const AcceptableKeysets& items)
{
    std::for_each(cbegin(items), cend(items), [&dst](auto&& item)
    {
        append(dst, item);
    });
}
