/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/database/check_configuration.hh"
#include "src/exception.hh"

#include <libfred/opcontext.hh>
#include <libfred/object/get_id_of_registered.hh>
#include <libfred/registrar/info_registrar.hh>

namespace Fred {
namespace Akm {
namespace Database {

void check_configuration(const KeysetMetadata& _keyset_meta)
{
    LibFred::OperationContextCreator ctx;

    try
    {
        const LibFred::InfoRegistrarData registrar_data =
            LibFred::InfoRegistrarByHandle(_keyset_meta.registrar).exec(ctx).info_registrar_data;

        const bool is_system_registrar = registrar_data.system.get_value_or(false);

        if (!is_system_registrar)
        {
            throw ConfigurationError{"keyset registrar is not system registrar"};
        }

        LibFred::get_id_of_registered<LibFred::Object_Type::contact>(ctx, _keyset_meta.tech_contact);
    }
    catch (const LibFred::InfoRegistrarByHandle::Exception& e)
    {
        std::string registrar = "registrar: '" + _keyset_meta.registrar + "'";
        if (e.is_set_unknown_registrar_handle())
        {
            throw ConfigurationError{registrar + " not found"};
        }
        throw ConfigurationError{registrar + " error: " + e.what()};
    }
    catch (const LibFred::UnknownObject&)
    {
        throw ConfigurationError{std::string("tech contact '") + _keyset_meta.tech_contact + "' not found"};
    }
    catch (const ConfigurationError&)
    {
        throw;
    }
    catch (const std::exception& e)
    {
        throw ConfigurationError{e.what()};
    }
    catch (...)
    {
        throw ConfigurationError{"unknown error"};
    }
}

} // namespace Fred::Akm::Database
} // namespace Fred::Akm
} // namespace Fred
