/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef KEYSET_HH_93B09BA8F2842F4A8A406D30A76C3FA2//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define KEYSET_HH_93B09BA8F2842F4A8A406D30A76C3FA2

#include <libstrong/type.hh>

#include <cstdint>
#include <string>
#include <set>

namespace Fred {
namespace Akm {

using DnskeyFlags = LibStrong::Type<std::uint32_t, struct DnskeyFlagsTag_,
      LibStrong::Collect<LibStrong::Skill::Comparable, LibStrong::Skill::Streamable>::SuperSkill>;
using DnskeyProto = LibStrong::Type<std::uint32_t, struct DnskeyProtoTag_,
      LibStrong::Collect<LibStrong::Skill::Comparable, LibStrong::Skill::Streamable>::SuperSkill>;
using DnskeyAlg = LibStrong::Type<std::uint32_t, struct DnskeyAlgTag_,
      LibStrong::Collect<LibStrong::Skill::Comparable, LibStrong::Skill::Streamable>::SuperSkill>;
using DnskeyPublicKey = LibStrong::Type<std::string, struct DnskeyPublicKeyTag_,
      LibStrong::Collect<LibStrong::Skill::Comparable, LibStrong::Skill::Streamable>::SuperSkill>;

struct Dnskey
{
    DnskeyFlags flags;
    DnskeyProto proto;
    DnskeyAlg alg;
    DnskeyPublicKey public_key;
};

std::string to_string(const Dnskey&);

bool operator<(const Dnskey&, const Dnskey&);
bool operator==(const Dnskey&, const Dnskey&);

using Keyset = std::set<Dnskey>;

} // namespace Fred::Akm
} // namespace Fred

#endif//KEYSET_HH_93B09BA8F2842F4A8A406D30A76C3FA2
